;--- SET TILE PALETTE -------------------------------------------------------
;--- HL = palette address ---------------------------------------------------
SET_TILE_PALETTE:
  push hl
  ld hl, VRAM_TILE_PALETTE
  call SET_VDP_POINTER
  pop hl
  ld bc, 32
  call LOAD_TO_VRAM
  ret
;--- SET SPRITE PALETTE -----------------------------------------------------
;--- HL = palette address --------------------------------------------------- 
SET_SPRITE_PALETTE:
  push hl
  ld hl, VRAM_SPRITE_PALETTE
  call SET_VDP_POINTER
  pop hl
  ld bc, 32
  call LOAD_TO_VRAM
  ret
;--- CLEAR TILE PALETTE -----------------------------------------------------
CLEAR_TILE_PALETTE:
  ld hl, VRAM_TILE_PALETTE
  call SET_VDP_POINTER
  ld b, 32
  ld a, 0
CLEAR_TILE_PALETTE_LOOP:
  out ($BE), a
  dec b
  jr NZ, CLEAR_TILE_PALETTE_LOOP
  ret
;--- FADE IN ----------------------------------------------------------------
;--- A = brightness   IX = address of palette -------------------------------
;--- D = how many times to fade   E = slowdown ------------------------------
FADE_IN:
  push af                    ; save everything that is important
  push de
  push ix
    ld hl, DUMMY             ; write the brightness value into (DUMMY)
    ld (HL), a
  call CREATE_TILE_PALETTE   ; not create the tile palette and display it
    ld d, e                  ; wait a few VWAITs
  call VWAIT                 
  pop ix                     ; get the important stuff back
  pop de
  pop af
  inc a                      ; light the palette up
  dec d                      ; next step
  jr NZ, FADE_IN
  ret
;--- FADE OUT ---------------------------------------------------------------
;--- A = brightness   IX = address of palette -------------------------------
;--- D = how many times to fade   E = slowdown ------------------------------
FADE_OUT:
  push af                    ; save everything that is important
  push de
  push ix
    ld hl, DUMMY             ; write the brightness value into (DUMMY)
    ld (HL), a
  call CREATE_TILE_PALETTE   ; not create the tile palette and display it
    ld d, e                  ; wait a few VWAITs
  call VWAIT
  pop ix                     ; get the important stuff back
  pop de
  pop af
  dec a                      ; darken the palette
  dec d
  jr NZ, FADE_OUT
  ret
;--- SET PALETTE RGB VALUE --------------------------------------------------
;--- HL = palette entry address ---------------------------------------------
;--- B = RED   C = GREEN   D = BLUE -----------------------------------------
SET_PAL_RGB:
  ld a, c               ; 00000ggg
  rl a                  ; 0000ggg0
  rl a                  ; 000ggg00
  rl a                  ; 00ggg000
  rl a                  ; 0ggg0000
  add a, b              ; 0ggg0rrr
  rl a                  ; ggg0rrr0
  ld (HL), a
  inc hl
  ld a, d               ; 00000bbb
  rl a                  ; 0000bbb0
  ld (HL), a
  inc hl
  ret
;--- GET PALETTE RGB VALUE --------------------------------------------------
;--- HL = palette entry address ---------------------------------------------
;--- Output in B = RED   C = GREEN   D = BLUE -------------------------------
GET_PAL_RGB:
  push hl
  ld a, (HL)               ; get red value
  and 00001110b
  rr a
  ld b, a

  ld a, (HL)               ; get green value
  and 11100000b
  rr a
  rr a
  rr a
  rr a
  rr a
  ld c, a

  inc hl                   ; get blue value
  ld a, (HL)
  and 00001110b
  rr a
  ld d, a
  pop hl
  ret
;--- CREATE TILE PALETTE ----------------------------------------------------
; This procedure creates a brighter or darker
; palette and then displays it.
;--- IX = source palette   (DUMMY) = brightness -----------------------------
CREATE_TILE_PALETTE:
    ld hl, DUMMY_PALETTE        ; copy the palette into RAM so
    ld c, 32                    ; that it can be altered and
  call COPY_MEM                 ; to ensure that the specified
                                ; palette isn't changed

  ld ix, DUMMY
  ld hl, DUMMY_PALETTE
  ld a, 16
PAL_LOOP:
  push af

  call GET_PAL_RGB       ; get the rgb values
  ld a, b                ; change red value
  add a, (IX)
  jp P, NOT_LESS_THAN_0R
  ld a, 0
NOT_LESS_THAN_0R:
  cp 7
  jp C, NOT_MORE_THAN_7R
  ld a, 7
NOT_MORE_THAN_7R:
  ld b, a                ; set red value

  ld a, c                ; change green value
  add a, (IX)
  jp P, NOT_LESS_THAN_0G
  ld a, 0
NOT_LESS_THAN_0G:
  cp 7
  jp C, NOT_MORE_THAN_7G
  ld a, 7
NOT_MORE_THAN_7G:
  ld c, a                ; set green value

  ld a, d                ; change blue value
  add a, (IX)
  jp P, NOT_LESS_THAN_0B
  ld a, 0
NOT_LESS_THAN_0B:
  cp 7
  jp C, NOT_MORE_THAN_7B
  ld a, 7
NOT_MORE_THAN_7B:
  ld d, a                ; set blue value

  call SET_PAL_RGB       ; set the rgb values

  pop af
  dec a                  ; next color
  jr NZ, PAL_LOOP

  ld hl, DUMMY_PALETTE   ; now display the palette
  call SET_TILE_PALETTE
  ret
;--- FADE IN ONE STEP -------------------------------------------------------
;--- IX = palette address   B = fading slowdown -----------------------------
;--- (DUMMY3) = slowdown value   (DUMMY) = current brightness value ---------
FADE_IN_ONE_STEP:
  ld a, (DUMMY3)                      ; don't fade in until DUMMY3 = b
  inc a
  ld (DUMMY3), a
  cp b
  ret NZ

  ld a, 0                             ; reset DUMMY3
  ld (DUMMY3), a

  ld a, (DUMMY)                       ; fade in,
  cp 0                                ; but check first if it
  ret Z                               ; has already been done
  inc a
    ld (DUMMY), a
  call CREATE_TILE_PALETTE
  ret
